/**
 * Copyright: (C) 1987-2005 Rene Stoeckel (RST). Licence: This program is
 * private property. Licence: Use it with explicit permission of the Author
 * only. Contact: rene.stoeckel-at-web.de
 */

// Created on 12.06.2005 by Rene Stoeckel.
//
// $Id:$
package iterate;

import mandel.StopWatch;

public class MandelBenchmark
{
	/** Compares JNI and native java speed on calculation the mandelbrot set. */
	public static void main(String[] args)
	{
		int HEIGHT = 45;
		int WIDTH = 120;
		int iterations = 50000;
		double x1 = -2.0;
		double x2 = 1.0;
		double y1 = -1.5;
		double y2 = 1.5;
		int buffer[] = new int[WIDTH * HEIGHT];

		System.out.println("Testing mandelbrot iteration implemented in java.");
		for (int n = 0; n < 5; n++)
		{
			StopWatch.start();
			IterateJava.iterate(x1, x2, y1, y2, iterations, WIDTH, HEIGHT, buffer);
			StopWatch.stop();
		}

		IterateJNI jni = new IterateJNI();

		System.out.println("Testing optimized C version of iteration via JNI, done by RST.");
		for (int n = 0; n < 5; n++)
		{
			StopWatch.start();
			jni.iterate(x1, x2, y1, y2, iterations, WIDTH, HEIGHT, buffer);
			StopWatch.stop();
		}

		System.out.print("Generated Mandelbrot:");
		IterateJava.iterate(x1, x2, y1, y2, iterations, WIDTH, HEIGHT, buffer);

		for (int py = 0; py < HEIGHT; py++)
		{
			System.out.println();
			for (int px = 0; px < WIDTH; px++)
				System.out.print((char) (32 + buffer[px * HEIGHT + py]));
		}

		System.out.println();
	}
}