package mandel;
import iterate.IterateJava;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

//Created on 04.05.2004 by RST.
//$Id: GridManager.java,v 1.1 2004/06/02 21:58:36 rst Exp $

public class GridManager
{
	public static class Grid
	{

		// valid for all grids.
		static double dx;
		static double dy;
		static int width;
		static int height;
		static int gridsize = 100;

		public Grid()
		{
			points = new int[gridsize * gridsize];
		}

		public Grid(double xmin, double ymin)
		{
			points = new int[gridsize * gridsize];
			x1 = xmin;
			y1 = ymin;
		}

		boolean rendered;
		double x1;
		double y1;
		int points[];

		BufferedImage bi;

		public void checkForRendering()
		{

			if (rendered)
				return;

			IterateJava.iterateJava2(x1, dx, y1, dy, 250, gridsize, gridsize, points);
			rendered = true;
			/*
			 * bi = new BufferedImage(gridsize, gridsize,
			 * BufferedImage.TYPE_INT_ARGB); int n = 0; for (int x = 0; x <
			 * gridsize; x++) for (int y = 0; y < gridsize; y++) bi.setRGB(x, y,
			 * getColor(points[n++]));
			 */
		}
	}

	private Preset preset;

	private int xgrids = 1;
	private int ygrids = 1;

	private Grid grids[][];

	/** Updates all grids. */
	public void updateGridCoords()
	{
		Grid.dx = (preset.xmax - preset.xmin) / Grid.width;
		Grid.dy = (preset.ymax - preset.ymin) / Grid.height;

		double gx = Grid.dx * Grid.gridsize;
		double gy = Grid.dy * Grid.gridsize;

		double x1 = preset.xmin;

		for (int x = 0; x < xgrids; x++)
		{
			double y1 = preset.ymin;
			for (int y = 0; y < ygrids; y++)
			{
				grids[x][y].y1 = y1;
				grids[x][y].x1 = x1;
				y1 += gy;
				grids[x][y].rendered = false;
			}
			x1 += gx;
		}
	}

	/** Initialize the grid structure. */
	public void init(int width, int height, Preset p)
	{
		preset = p;

		Grid.width = width;
		Grid.height = height;

		xgrids = width / Grid.gridsize + 1;
		ygrids = height / Grid.gridsize + 1;

		grids = new Grid[xgrids][ygrids];

		for (int x = 0; x < xgrids; x++)
		{
			for (int y = 0; y < ygrids; y++)
			{
				grids[x][y] = new Grid();
			}
		}
		updateGridCoords();
	}

	/** Scrolls the view. */
	public void moveview(int dx, int dy)
	{
		double gx = Grid.dx * Grid.gridsize;
		double gy = Grid.dy * Grid.gridsize;

		preset.xmin += dx * gx;
		preset.ymin += dy * gy;

		double x1 = preset.xmin;

		Grid newgrids[][] = new Grid[xgrids][ygrids];

		for (int x = 0; x < xgrids; x++)
		{
			double y1 = preset.ymin;

			for (int y = 0; y < ygrids; y++)
			{
				int fromx = x + dx;
				int fromy = y + dy;

				// copy the grid
				if (fromx >= 0 && fromx < xgrids && fromy >= 0 && fromy < ygrids)
					newgrids[x][y] = grids[fromx][fromy];
				else
				{
					// create a new grid
					newgrids[x][y] = new Grid();
					newgrids[x][y].y1 = y1;
					newgrids[x][y].x1 = x1;
				}
				y1 += gy;
			}
			x1 += gx;
		}
		grids = newgrids;
	}

	/** Zoom in the scroll view. */
	public void zoomin(double fraction)
	{

		double xlen = (preset.xmax - preset.xmin);
		double ylen = (preset.ymax - preset.ymin);

		double newxlen = fraction * xlen;
		double newylen = fraction * ylen;

		double xdiff = (newxlen - xlen) / 2;
		double ydiff = (newylen - ylen) / 2;

		// translate the start and endpoints
		preset.xmin -= xdiff;
		preset.xmax += xdiff;

		preset.ymin -= ydiff;
		preset.ymax += ydiff;
	}

	/** Draws a grid to the graphics canvas. */
	public void drawGrid(Graphics g, int x1, int y1, int yreverse, Grid grid)
	{
		System.out.println("drawgrid!");
		for (int x = 0; x < Grid.gridsize; x++)
		{
			for (int y = 0; y < Grid.gridsize; y++)
			{
				Color c;

				grid.checkForRendering();

				//g.drawImage(grid.bi, x1, y1, null);
				c = RainbowPalette.getColorPlus(grid.points[x * grid.gridsize + y]);

				if (x == 0 || y == 0)
				{
					c = Color.black;
				}
				g.setColor(c);
				g.drawLine(x1 + x, yreverse - (y1 + y), x1 + x, yreverse - (y1 + y));

			}
		}
	}

	/** Draws all grids. */
	public void drawTo(Graphics g, int yreverse)
	{
		int x1 = 0;
		for (int x = 0; x < xgrids; x++)
		{
			int y1 = 0;
			for (int y = 0; y < ygrids; y++)
			{
				drawGrid(g, x1, y1, yreverse, grids[x][y]);
				y1 += Grid.gridsize;
			}
			x1 += Grid.gridsize;
		}
	}
}