package mandel;
import iterate.IterateJava;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

public class MandelCanvas extends Canvas
{

	MandelMain mandel = null;

	private boolean first_time; // Is this the first time painting?
	private RainbowPalette rp; // Special Palette class
	private ColorMapper gp; // Special Palette class
	private int colorDepth; // Color depth "constant"
	private int iterations; // Iteration count "constant"
	private int resolution; // Resolution size "constant"
	private Image img; // Offscreen copy of image
	private int markx1, marky1, markx2, marky2;
	private boolean markOn;


	// The MandelCanvas constructor constructs the object.  All it
	// really does is initialize variables.
	public MandelCanvas(MandelMain m)
	{
		this.mandel = m;
		addMouseListener(new MouseAdapter()
		{
			public void mousePressed(MouseEvent e)
			{
				markx1 = e.getX();
				marky1 = e.getY();
				markOn = true;
			}

			public void mouseReleased(MouseEvent e)
			{
				markOn = false;

				int d = getWidth();
				int h = getHeight();

				System.out.println("d=" + d + ", h=" + h);

				int x1 = Math.min(markx1, markx2);
				int y1 = getHeight() - Math.min(marky1, marky2);

				int x2 = Math.max(markx1, markx2);
				int y2 = getHeight() - Math.max(marky1, marky2);

				repaint();
				System.out.println("new rendering: " + x1 + ", " + y1 + ", " + x2 + ", " + y2);
				// mirror y coordinate
				//mandel.zoom(x1 / d, y2 / h, x2 / d, y1 / h);
				mandel.zoom(x1, x2, y1, y2, d, h);
			}

		});

		addMouseMotionListener(new MouseMotionAdapter()
		{
			public void mouseDragged(MouseEvent e)
			{
				markx2 = e.getX();
				marky2 = e.getY();
				repaint();
			}
		});

		addKeyListener(new KeyAdapter()
		{
			public void keyPressed(KeyEvent e)
			{
				switch (e.getKeyCode())
				{
					case KeyEvent.VK_H :
						System.out.println("Help!");
						break;
				}
			}
		});

		first_time = true;
		// Initialize palettes;
		rp = new RainbowPalette();
		gp = new ColorMapper();
		// Set "constants":  If you build different palettes or wish
		// to increase the size of the canvas, you can change these
		// values to suit, but don't forget to update other dependant
		// items!
		colorDepth = 256;
		iterations = 1000;
		resolution = 300;
		
		this.requestFocus();
	} // MandelCanvas constructor+
	

	public void paint(Graphics g)
	{
		// First time the canvas is loaded, print a blurb:
		if (first_time)
		{
			about();
			first_time = false;
		} 
		else
			g.drawImage(img, 0, 0, 300, 300, null);

		g.setColor(Color.black);
		g.drawRect(0, 0, 299, 299);

		if (markOn)
		{
			int x1 = markx1 < markx2 ? markx1 : markx2;
			int y1 = marky1 < marky2 ? marky1 : marky2;
			int d = Math.abs(markx1 - markx2);
			int h = Math.abs(marky1 - marky2);
			g.setColor(Color.white);
			g.drawRect(x1, y1, d, h);
		}

	}

	public void setIterations(int iter)
	{
		this.iterations = iter;
	}

	/** 
	 * Update graphics without clear. 
	 */
	public void update(Graphics g)
	{
		paint(g);
	}

	// drawError draws an error message string to the canvas.
	public void drawError(String str)
	{
		Graphics g1 = getGraphics();
		img = createImage(300, 300);
		Graphics g2 = img.getGraphics();
		g1.setColor(Color.white);
		g1.fillRect(0, 0, 300, 300);
		g1.setColor(Color.black);
		g1.drawString(str, 10, 150);
		g2.setColor(Color.white);
		g2.fillRect(0, 0, 300, 300);
		g2.setColor(Color.black);
		g2.drawString(str, 10, 150);
		repaint();
	} // drawError()

	// about() prints the "about" information to the canvas:
	public void about()
	{
		Graphics g1 = getGraphics();
		img = createImage(300, 300);
		Graphics g2 = img.getGraphics();
		g1.setColor(Color.gray);
		g1.fillRect(0, 0, 300, 300);
		g1.setColor(Color.black);
		g1.setFont(new Font("Helvetica", Font.BOLD, 16));

		drawShadowString(g1, "Mandelbrot Madness JAVA! 1.1", 10, 100, Color.yellow);

		g1.setFont(new Font("Helvetica", Font.PLAIN, 12));

		drawShadowString(g1, "by Jeffrey T. Darlington", 10, 120, Color.yellow);
		drawShadowString(g1, "(c) Copyright 1998.  All rights reserved.", 10, 135, Color.yellow);
		drawShadowString(g1, "Choose a Preset View or enter your own values", 10, 170, Color.yellow);
		drawShadowString(g1, "into the X and Y value boxes.  Then choose", 10, 185, Color.yellow);
		drawShadowString(g1, "a color palette and click the Render button.", 10, 200, Color.yellow);

		g2.setColor(Color.gray);
		g2.fillRect(0, 0, 300, 300);

		g2.setColor(Color.yellow);
		g2.setFont(new Font("Helvetica", Font.BOLD, 16));
		drawShadowString(g2, "Mandelbrot Madness JAVA! 1.1", 10, 100, Color.yellow);
		g2.setFont(new Font("Helvetica", Font.PLAIN, 12));
		drawShadowString(g2, "by Jeffrey T. Darlington", 10, 120, Color.yellow);
		drawShadowString(g2, "(c) Copyright 1998.  All rights reserved.", 10, 135, Color.yellow);
		drawShadowString(g2, "Choose a Preset View or enter your own values", 10, 170, Color.yellow);
		drawShadowString(g2, "into the X and Y value boxes.  Then choose", 10, 185, Color.yellow);
		drawShadowString(g2, "a color palette and click the Render button.", 10, 200, Color.yellow);
	} // about()

	private static void drawShadowString(Graphics g1, String s, int x, int y, Color c)
	{
		g1.setColor(Color.black);
		g1.drawString(s, x + 2, y - 2);
		g1.drawString(s, x + 1, y - 2);
		g1.drawString(s, x + 1, y - 1);

		g1.setColor(c);
		g1.drawString(s, x, y);
		g1.drawString(s, x, y + 1);
	}

	// drawMandel() does the actual Mandelbrot calculations and draws the image.
	// It gets the x and y bounds, and which palette to use.
	public void drawMandel(double xmin, double xmax, double ymin, double ymax, boolean rp_b)
	{
		// Calculate the x and y gaps:
		//double xgap = (xmax - xmin) /  resolution;
		//double ygap = (ymax - ymin) /  resolution;
		int pixel_val; // The index value for the color of a certain pixel
		Color pixel_color; // Its corresponding color
		// Get the graphics context:
		// Graphics g1 = getGraphics();
		// Build new offscreen image and get its graphics context:
		img = createImage(resolution, resolution);
		Graphics g2 = img.getGraphics();

		int buffer[] = new int[resolution * resolution];

		IterateJava.iterate(xmin, xmax, ymin, ymax, iterations, resolution, resolution, buffer);

		System.out.println("done!");

		// Now we draw some numbers:
		for (int i = 0; i <= resolution - 1; i++)
		{
			for (int j = 0; j <= resolution - 1; j++)
			{
				// Get the pixel value:
				if ((pixel_val = buffer[i * resolution + j]) == iterations)
					pixel_val = 0;
				else
					pixel_val %= colorDepth;

				// Determine its color:
				if (rp_b)
					pixel_color = rp.getColor(pixel_val);
				else
					pixel_color = gp.getColor(pixel_val);
				
				g2.setColor(pixel_color);
				g2.drawLine(i, resolution - 1 - j, i, resolution - j);
			}
		}
		repaint();
	}
}